# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"DEFAULT_STRIDE": "00_core.ipynb",
         "DEFAULT_THRESHOLD": "00_core.ipynb",
         "NNSplit": "00_core.ipynb",
         "Tokenizer": "01_tokenizer.ipynb",
         "SoMaJoTokenizer": "01_tokenizer.ipynb",
         "DATA_DIRECTORY": "02_utils.ipynb",
         "CUT_LENGTH": "02_utils.ipynb",
         "DEVICE": "02_utils.ipynb",
         "text_to_id": "02_utils.ipynb",
         "id_to_text": "02_utils.ipynb",
         "MAX_N_SENTENCES": "03_train.ipynb",
         "REMOVE_DOT_CHANCE": "03_train.ipynb",
         "LOWERCASE_START_CHANCE": "03_train.ipynb",
         "MIN_LENGTH": "03_train.ipynb",
         "N_CUTS": "03_train.ipynb",
         "label_paragraph": "03_train.ipynb",
         "generate_data": "03_train.ipynb",
         "fast_iter": "03_train.ipynb",
         "prepare_data": "03_train.ipynb",
         "Network": "03_train.ipynb",
         "loss": "03_train.ipynb",
         "train_from_tensors": "03_train.ipynb",
         "train_from_directory": "03_train.ipynb",
         "store_model": "03_train.ipynb"}

modules = ["__init__.py",
           "tokenizer.py",
           "utils.py",
           "train.py"]

doc_url = "https://bminixhofer.github.io/nnsplit/"

git_url = "https://github.com/bminixhofer/nnsplit/tree/master/"

def custom_doc_links(name): return None
